!!2CO2-Ar PES
!!Distances in Angstrom and energies in Hartrees

  
module GP_variables_2CO2_Ar
  implicit none
  private
  !User constants
  !! ==== Low quality options ====
  !character (len=10) :: qualityString="lQ_" ! hQ or lQ for high/low quality
  !integer :: nTraining_A=1105    ! choose from: 1105
  !integer :: nTraining_B=1000    ! choose from 1000
  !integer :: nTraining_C=1481    ! choose from 1481,1098, 150 

  !! ====High quality options ====
  character (len=10), public :: qualityString="hQ_" ! hQ or lQ for high/low quality
  integer, public :: nTraining_A=1000     ! choose from: 1000
  integer, public :: nTraining_B=1000    ! choose from 1000
  integer, public :: nTraining_C=1400     ! choose from 1400, 1000 
  

  !System constants
  integer, public :: nDim=15   !Number of inter-atomic distances
  integer, public :: nPerms=8


  !global variables
  double precision, allocatable, public :: alpha_A (:), lScale_A(:), &
       xTraining_A(:,:), xTrainingPerm_A(:,:,:), &
       alpha_B (:), lScale_B(:), xTraining_B(:,:), xTrainingPerm_B(:,:,:), &
       alpha_C (:), lScale_C(:), xTraining_C(:,:), xTrainingPerm_C(:,:,:)
  double precision, public:: expVar_A,NuggVar_A, gpEmax_A, &
       expVar_B,NuggVar_B,gpEmax_B, &
       expVar_C,NuggVar_C, gpEmax_C

  !chemical constants
  double precision, parameter, public :: lCO = 1.1632D0
  double precision, parameter, public :: AngToBohr =  1.8897259885789D0

  !tolerances
  double precision, parameter, public :: distTol = 1d-15
  
end module GP_variables_2CO2_Ar

module perm_array
  implicit none
  private
  integer, allocatable, public:: perm(:,:)
end module perm_array


module GP_variables_Asympt_2CO2_Ar
  implicit none
  private
  !! ==== Low quality options ====
  !character (len=10) :: qualityString="lQ_" ! hQ or lQ for high/low quality
  !integer :: nTraining_A_Energy=1002! choose from 1002
  !integer :: nTraining_D_Energy=300! choose from 239, 300
  !integer :: nTraining_E_Energy=700! choose from 300, 700

  !! ==== High quality options ====
  character (len=10), public :: qualityString="hQ_" ! hQ or lQ for high/low quality
  integer, public :: nTraining_A_Energy=250! choose from 250
  integer, public :: nTraining_D_Energy=300! choose from 300
  integer, public :: nTraining_E_Energy=400! choose from 300, 400

  
  integer, public :: nAngles=6 !Number of angular co-ordinates for the power law region

  double precision, allocatable, public ::   alpha_A_Energy(:), lScale_A_Energy(:), xTraining_A_Energy(:,:), &
       xTrainingPerm_A_Energy(:,:,:)
  double precision, public::  expVar_A_Energy, NuggVar_A_Energy
  
  double precision, allocatable, public ::   alpha_D_Energy(:), lScale_D_Energy(:), xTraining_D_Energy(:,:), &
       xTrainingPerm_D_Energy(:,:,:)
  double precision, public::  expVar_D_Energy, NuggVar_D_Energy

  double precision, allocatable, public ::   alpha_E_Energy(:), lScale_E_Energy(:), xTraining_E_Energy(:,:), &
       xTrainingPerm_E_Energy(:,:,:)
  double precision, public::  expVar_E_Energy, NuggVar_E_Energy

  
end module GP_variables_Asympt_2CO2_Ar



module RegionA_Options_2CO2_Ar
  implicit none
  private
  double precision, public :: RShortMax=4.501d0, RMedMax=5.0001d0, RLongMax=9.30001d0, r13ref=7.4d0
  integer, public :: alpha=-6, beta=-7
end module RegionA_Options_2CO2_Ar

module RegionD_Options_2CO2_Ar
  implicit none
  private
  double precision, public :: RShortMax=4.501d0, RMedMax=5.0001d0, RLongMax=8.500000d0, r12ref=8.5d0
  integer, public :: alpha=-4, beta=-6
end module RegionD_Options_2CO2_Ar

module RegionE_Options_2CO2_Ar
  implicit none
  private
  double precision, public :: RShortMax=4.501d0, RMedMax=5.0101d0, RLongMax=8.500000d0, r23ref=7.4d0
  integer, public :: alpha=-4, beta=-6
end module RegionE_Options_2CO2_Ar

