module Overall_PES_2CO2_Ar
  use Assert_Module
  use PES_GP_Module, only: PES_GP
  use AnglesStructures, only: Angles
  use CentreToCentreTuple_Module, only: CentreToCentreTuple
  use GP_variables_2CO2_Ar, only: nDim,  distTol
  use permutations, only: fixpermutations
  use compDist_2CO2_Ar, only: computeDistances
  use compAngles_2CO2_Ar, only:computeAngles
  use RegionA_Extrapolate_Module, only: RegionA_Extrapolate
  use RegionD_Extrapolate_Module, only: RegionD_Extrapolate
  use RegionE_Extrapolate_Module, only: RegionE_Extrapolate
  use checkRegion_2CO2_Ar, only: isRegionE, isRegionD, isRegionA
  implicit none

  !!====Public interface================================================
  public PES_2CO2_Ar
  !!====================================================================
  
  private


contains

  
double precision function PES_2CO2_Ar( rab )
  !! Takes in rab in Angstrom
  double precision, dimension(nDim) :: rab ! Not intent(in) as needs to be permuted

  rab = fixPermutations( rab)
  
  if (  isRegionE( rab)   ) then
     PES_2CO2_Ar = computeRegionE( rab)
     return
  endif 

  if (  isRegionD( rab )  ) then
     PES_2CO2_Ar = computeRegionD( rab)
     return
  endif 
  
  if (  isRegionA( rab )  ) then
     PES_2CO2_Ar = computeRegionA( rab)
     return
  endif 

  print *,rab
  print *,'Never reach here!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
  STOP 100
     
end function PES_2CO2_Ar


double precision function computeRegionA( rab )
  use RegionA_Options_2CO2_Ar, only: RShortMax
  double precision, dimension(nDim), intent(in) :: rab

  double precision  scale
  type( Angles ) :: myAngles
  type(CentreToCentreTuple) :: rnew
  double precision, dimension(nDim) ::  rabnew
  
  if(  minval( rab(1:9) )  <  RShortMax ) then
     computeRegionA = RegionA_Extrapolate( rab) 
     return
  endif
     
  ! ==== Ok then, we'll have to scale because r12 is too big====
  scale =RShortMax/rab(5);

  rnew = CentreToCentreTuple(scale*rab(5), scale*rab(11),scale*rab(14))
  myAngles = computeAngles(rab)
  rabnew = computeDistances( rnew, myAngles)

  call assertEqual_double( rabnew(5),RShortMax , 1d-8, &
       "Scale failed to map to correct min dist - Reg A")

  computeRegionA = scale**9 * regionA_Extrapolate( rabnew) 
end function computeRegionA


double precision function computeRegionD( rab )
  use RegionD_Options_2CO2_Ar, only: RShortMax
  double precision, dimension(nDim), intent(in) :: rab

  double precision  scale
  type( Angles ) :: myAngles
  type(CentreToCentreTuple) :: rnew
  double precision, dimension(nDim) ::  rabnew

  if( minval(rab(10:12) )< RShortMax ) then
     computeRegionD = RegionD_Extrapolate( rab) 
     return
  endif

  ! ==== Need to scale because r13 is too big====
  scale =RShortMax/rab(11)

  rnew = CentreToCentreTuple(scale*rab(5), scale*rab(11), scale*rab(14))
  myAngles = computeAngles( rab)
  rabnew = computeDistances( rnew, myAngles)

  call assertEqual_double( rabnew(11),RShortMax , 1d-8, &
       "Scale failed to map to correct min dist- Reg D")

  computeRegionD = scale**9 * regionD_Extrapolate( rabnew) 
end function computeRegionD


double precision function computeRegionE( rab )
  use RegionE_Options_2CO2_Ar, only: RShortMax
  double precision, dimension(nDim), intent(in) :: rab

  double precision  scale
  type( Angles ) :: myAngles
  type(CentreToCentreTuple) :: rnew
  double precision, dimension(nDim) ::  rabnew

  if( minval(rab(10:12) )< RShortMax ) then
     computeRegionE = RegionE_Extrapolate( rab) 
     return
  endif

  ! ==== Need to scale because r13 is too big====
  scale = RShortMax/ rab(11)

  rnew = CentreToCentreTuple(scale*rab(5), scale*rab(11), scale*rab(14))
  myAngles = computeAngles( rab)
  rabnew = computeDistances( rnew, myAngles)

  call assertEqual_double( rabnew(11),RShortMax , 1d-8, &
       "Scale failed to map to correct min dist - RegE")

  computeRegionE = scale**9 * regionE_Extrapolate( rabnew) 
end function computeRegionE


end module Overall_PES_2CO2_Ar
