module compDist_2CO2_Ar
  use GP_variables_2CO2_Ar, only: nDim
  use CentreToCentreTuple_Module, only: CentreToCentreTuple
  use AnglesStructures, only: Angles
  implicit none
  private
  public ::computeDistances, computeDistances_withCO2slide,compX, compThetaHat, compThetaTilde,&
       findSmallestDeltaX2CO2,  findSmallestDeltaXCO2Ar,findDeltaX,sqLength, computeCtoO1, &
       computeCtoO3


contains
  
  function computeDistances(RIn, anglesIn) result( rab )
    type( CentreToCentreTuple ), intent(in):: RIn
    type( Angles ), intent(in):: anglesIn
    double precision, dimension(nDim)::  rab
    
    type( Angles ):: anglesWithCos
    double precision :: O1(3)=0.0,O2(3)=0.0,C1(3)=0.0,O3(3)=0.0,C2(3)=0.0,O4(3)=0.0,Ar(3)=0.0
    double precision :: cosTheta13

    anglesWithCos = computeSinAndCos( anglesIn )
    
    !==== CO2 1 ====
    !C is already at the origin
    O1 = computeCtoO1( anglesWithCos )
    O2 = -O1
    
    !==== CO2 2 ====
    C2(3)= RIn%r12;
    O3 =  computeCtoO3( anglesWithCos ) + C2
    O4 = -computeCtoO3( anglesWithCos ) + C2
    
    !==== Ar ====
    cosTheta13 = (RIn%r12**2 + RIn%r13**2 - RIn%r23**2) / 2.0/RIn%r12/RIn%r13;
    Ar(1) = RIn%r13 * sqrt( abs(1.0 - cosTheta13*cosTheta13));
    Ar(3) = RIn%r13 * cosTheta13;

    rab = positionsToInteratomicDistances( O1, O2, O3, O4, C1, C2, Ar)
    
  end function computeDistances

  
  function computeCtoO1( anglesIn) result (O1)
    use GP_variables_2CO2_Ar, only: lCO
    type( Angles ), intent(in):: anglesIn
    double precision, dimension(3)::  O1

    O1 = lCO * (/ anglesIn%sinBeta1 * anglesIn%cosAlpha1, &
                  anglesIn%sinBeta1 * anglesIn%sinAlpha1, &
                  anglesIn%cosBeta1 /)
    
  end function computeCtoO1


  function computeCtoO3( anglesIn) result (O3)
    use GP_variables_2CO2_Ar, only: lCO
    type( Angles ), intent(in):: anglesIn
    double precision, dimension(3)::  O3

    O3 = lCO * (/ anglesIn%sinBeta2 * anglesIn%cosAlpha2, &
                  anglesIn%sinBeta2 * anglesIn%sinAlpha2, &
                  anglesIn%cosBeta2 /)
    
  end function computeCtoO3

  function computeSinAndCos( anglesIn ) result (anglesOut)
    type( Angles ), intent(in):: anglesIn
    type( Angles ):: anglesOut

    anglesOut = anglesIn
    
    anglesOut%cosAlpha1 = cos( anglesIn%alpha1 )
    anglesOut%sinAlpha1 = sin( anglesIn%alpha1 )
    anglesOut%cosAlpha2 = cos( anglesIn%alpha2 )
    anglesOut%sinAlpha2 = sin( anglesIn%alpha2 )
    anglesOut%cosBeta1  = cos( anglesIn%beta1 )
    anglesOut%sinBeta1  = sin( anglesIn%beta1 )
    anglesOut%cosBeta2  = cos( anglesIn%beta2 )
    anglesOut%sinBeta2  = sin( anglesIn%beta2 )

  end function computeSinAndCos


  function positionsToInterAtomicDistances( O1, O2, O3, O4, C1, C2, Ar) result (rab)
    double precision, intent(in), dimension(3) :: O1, O2, O3, O4, C1, C2, Ar
    double precision, dimension(nDim)::  rab
    
    rab(1) = sqrt( sqLength(O1,O3)  )
    rab(2) = sqrt( sqLength(O1,C2)  )
    rab(3) = sqrt( sqLength(O1,O4)  )

    rab(4) = sqrt( sqLength(C1,O3)  )
    rab(5) = sqrt( sqLength(C1,C2)  )
    rab(6) = sqrt( sqLength(C1,O4)  )

    rab(7) = sqrt( sqLength(O2,O3)  )
    rab(8) = sqrt( sqLength(O2,C2)  )
    rab(9) = sqrt( sqLength(O2,O4)  )
    
    rab(10) = sqrt(  sqLength(Ar,O1)  )
    rab(11) = sqrt(  sqLength(Ar,C1)  )
    rab(12) = sqrt(  sqLength(Ar,O2)  )
    
    rab(13) = sqrt(  sqLength(Ar,O3)  )
    rab(14) = sqrt(  sqLength(Ar,C2)  )
    rab(15) = sqrt(  sqLength(Ar,O4)  )

  end function positionsToInterAtomicDistances

  
  function computeDistances_withCO2slide(RIn, anglesIn,cosThetaTilde,deltaX) result (rab)
    type( CentreToCentreTuple ), intent(in):: RIn
    type( Angles ), intent(in):: anglesIn
    double precision, dimension(nDim)::  rab
    
    type( Angles ):: anglesWithCos
    double precision  cosThetaTilde,deltaX, cosTheta13
    double precision :: O1(3)=0.0,O2(3)=0.0,C1(3)=0.0,O3(3)=0.0,C2(3)=0.0,O4(3)=0.0,Ar(3)=0.0

    anglesWithCos = computeSinAndCos( anglesIn )
    
    !==== CO2 1 ====
    !C is already at the origin
    O1 =  computeCtoO1( anglesWithCos )
    O2 = -O1
    
    !==== CO2 2 ====
    C2(1)= deltaX *  sqrt( abs(1.0 - cosThetaTilde*cosThetaTilde))
    C2(3)= Rin%r12 - deltaX * cosThetaTilde

    O3 =  computeCtoO3( anglesWithCos ) + C2
    O4 = -computeCtoO3( anglesWithCos ) + C2
    
    !==== Ar ====
    cosTheta13 = (Rin%r12**2 + Rin%r13**2 - Rin%r23**2) / 2.0/Rin%r12/Rin%r13
    Ar(1) = Rin%r13 * sqrt( abs(1.0 - cosTheta13*cosTheta13));
    Ar(3) = Rin%r13 * cosTheta13;
    
    rab = positionsToInteratomicDistances( O1, O2, O3, O4, C1, C2, Ar)

  end function computeDistances_withCO2slide
    
  
  double precision function compThetaHat( r12,  r13, r23,  x)
    double precision, intent(in):: r12,r13,r23,x
    compThetaHat= (r23*r23 - r13*r13)/ 2.0 / r12/ x;
    
  end function compThetaHat

  
  double precision function compThetaTilde( r12,  r13,  x)
    double precision, intent(in):: r12,r13,x
    compThetaTilde= (r12*r12 + x*x - r13*r13/4.0)/ 2.0 / r12/ x;
    
  end function compThetaTilde
  
  
  double precision function compX( r12, r13, r23)
    double precision, intent(in):: r12,r13,r23
    compX =   0.5d0* sqrt( 2.0d0*( r13*r13 + r23*r23) - r12*r12);
  end function compX

  
  
  double precision function findDeltaX( bondVector_SlidingAtom , bondVector_StationaryAtom,&
       centreToCentre, slideDirection, RTarget)
    double precision, intent(in):: bondVector_SlidingAtom(3) , bondVector_StationaryAtom(3), &
         centreToCentre(3), slideDirection(3), RTarget
    
    double precision originalAtomToAtom(3)
    double precision originalDistanceSqr
    double precision distanceAlongSlide, discriminant
    
    originalAtomToAtom = -bondVector_SlidingAtom + centreToCentre + bondVector_StationaryAtom
    originalDistanceSqr = dot_product(originalAtomToAtom,originalAtomToAtom)
    distanceAlongSlide = dot_product(slideDirection,originalAtomToAtom) 


    discriminant = distanceAlongSlide**2 - originalDistanceSqr   +   RTarget**2
    if( discriminant <0 ) then
       findDeltaX = 1.d6 !If the method fails then one of the other slides must be the minimum
    else
       findDeltaX =  -distanceAlongSlide  -  sqrt( distanceAlongSlide**2 - originalDistanceSqr &
            +   RTarget**2 )
    endif
  end function findDeltaX
  



  double precision function findSmallestDeltaX2CO2( CO1 , CO3, CC, slideDirection, RTarget)
    double precision, intent(in):: CO1(3) , CO3(3), CC(3), slideDirection(3), RTarget
    
    findSmallestDeltaX2CO2 = min( &
         findDeltaX( CO1  , CO3  , CC, slideDirection, RTarget), &
         findDeltaX( CO1  ,-CO3  , CC, slideDirection, RTarget), &
         findDeltaX( CO1  , 0*CO3, CC, slideDirection, RTarget), &
         findDeltaX(-CO1  , CO3  , CC, slideDirection, RTarget), &
         findDeltaX(-CO1  ,-CO3  , CC, slideDirection, RTarget), &
         findDeltaX(-CO1  , 0*CO3, CC, slideDirection, RTarget), &
         findDeltaX( 0*CO1, CO3  , CC, slideDirection, RTarget), &
         findDeltaX( 0*CO1,-CO3  , CC, slideDirection, RTarget), &
         findDeltaX( 0*CO1, 0*CO3, CC, slideDirection, RTarget) )
    
  end function findSmallestDeltaX2CO2


  double precision function findSmallestDeltaXCO2Ar( CToO3 , ArToC2, slideDirection, RTarget)
    double precision, intent(in):: CToO3(3) , ArToC2(3), slideDirection(3), RTarget
    double precision::  zeroVector(3)=[ double precision :: 0.0, 0.0 , 0.0]
    
    findSmallestDeltaXCO2Ar = min( &
         findDeltaX( CToO3   , zeroVector  , ArToC2 , slideDirection, RTarget), &
         findDeltaX(0*CToO3  , zeroVector  , ArToC2 , slideDirection, RTarget), &
         findDeltaX( -CToO3  , zeroVector  , ArToC2 , slideDirection, RTarget))
    
  end function findSmallestDeltaXCO2Ar
  

  double precision function sqLength( u,v)
    double precision, intent(in):: u(3),v(3)
    sqLength = (u(1)-v(1))**2 + (u(2)-v(2))**2 + (u(3)-v(3))**2
  end function sqLength

end module compDist_2CO2_Ar

