module inSituChecks
  use GP_variables_2CO2_Ar, only: nDim, nPerms
  use Overall_PES_2CO2_Ar, only: PES_2CO2_Ar
  use perm_array, only: perm !!! The permutation group that was loaded at initialisation
  use Overall_PES_2CO2_Ar, only: PES_2CO2_Ar
  use permutations, only: fixpermutations
  implicit none

  private

  !!====Public interface================================================
  public testPerms, arePermsTheSame
  !!====================================================================

  contains

    
subroutine arePermsTheSame(rab)
  double precision, dimension(nDim), intent(in) :: rab
  
  integer perm_count
  double precision, dimension(nDim):: rab1Corrected, rabPerm, rabAfter
  double precision :: toll = 1e-15
  integer :: i
  
  rab1Corrected = fixPermutations( rab )

  do perm_count=2,nperms

     do i=1,nDim
        rabPerm(i)  =  rab( perm( i, perm_count ) ) !! Carry out the permutation
     enddo

     rabAfter = fixPermutations( rabPerm )

     do i=1,nDim
        if ( abs(rab1Corrected(i) - rabAfter( i ) ) > toll) then 
           print *,'*********** PERMUTATIONS ARE DIFFERENT (Perm ',perm_count,')***************'
           print *,' rab  =',rab
           print *,' perm =',rabPerm

           print *,' first =',rab1Corrected
           print *,' error =',rabAfter
           STOP 100
        endif
     enddo
  enddo
end subroutine arePermsTheSame


subroutine testPerms(rab)
  double precision, dimension(nDim), intent(in) :: rab
  
  integer perm_count
  double precision, dimension(nDim):: rabPerm
  double precision, dimension(nPerms) :: EPerm
  double precision :: toll = 1e-8, toll2=1e-8, error2
  integer :: i

  call arePermsTheSame(rab)
  
  EPerm(1) = PES_2CO2_Ar( rab)
  
  do perm_count=2,nperms
     
     do i=1,nDim
        rabPerm(i)  =  rab( perm( i, perm_count ) ) !! Carry out the permutation
     enddo
     EPerm( perm_count) =  PES_2CO2_Ar( rabPerm)
     
     !! Catch large absolute or % differences
     if( abs(EPerm(1))<1e-17) then
        error2 = 0.0
     else
        error2  =  (EPerm(1) - EPerm(perm_count))  /  EPerm(1)
     endif
     
     if( abs(EPerm(1)-EPerm(perm_count))>toll .and. abs( error2 )>toll2) then
        print *,'****************** PERMUTATION ERROR (Perm ',perm_count,')********************'
        print *,' rab  =',rab
        print *,' perm =',rabPerm
        print *, 'E(Perm1)',EPerm(1), 'E(other perm)',EPerm(perm_count)
        print *,'Abs err:',abs(EPerm(1)-EPerm(perm_count)), &
             '  % err:', abs(error2)*100
        STOP 100
     endif
  enddo

end subroutine testPerms
  
  
end module inSituChecks
  

