module loadData_2CO2_Ar
  implicit none
  private
  public ::load_GP_Data_2CO2_Ar, load_GP_Asymp_Data_2CO2_Ar

contains


subroutine load_GP_Data_2CO2_Ar
  use GP_variables_2CO2_Ar ! Data structure - requires all variables
  double precision, allocatable::  xStar(:)
  integer i,j,k
  double precision :: dum
  character (len=300) :: filename
  integer, allocatable:: perm(:,:)
  CHARACTER(len=300) :: homedir,codedir

  allocate (alpha_A(nTraining_A), lScale_A(nDim), xTraining_A(nDim,nTraining_A) &
       ,xTrainingPerm_A(nDim,nPerms,nTraining_A), xStar(nDim),    perm(nDim,nPerms))
   allocate (alpha_B(nTraining_B), lScale_B(nDim), xTraining_B(nDim,nTraining_B) &
        ,xTrainingPerm_B(nDim,nPerms,nTraining_B))
   allocate (alpha_C(nTraining_C), lScale_C(nDim), xTraining_C(nDim,nTraining_C) &
        ,xTrainingPerm_C(nDim,nPerms,nTraining_C))

   CALL getenv("HOME", homedir)
   codedir=TRIM(homedir) // '/source/2CO2-Ar_PES/'

  !====Load hyperparameters====
  if(nTraining_A<1000) then
     write (filename, '( "TrainingData/",A3,"RegionA_HyperParams_Symm", I3.3, ".dat" )' )  qualityString, nTraining_A
  else
     write (filename, '( "TrainingData/",A3,"RegionA_HyperParams_Symm", I4.4, ".dat" )' )  qualityString,nTraining_A
  endif

  filename = trim(codedir) // filename
  open (unit = 7, file = filename)
  do i=1,nDim
     read (7,*) dum
     j=int(dum)
     read (7,*) lScale_A(j)
  end do
  read (7,*) expVar_A
  read (7,*) NuggVar_A
  read (7,*) gpEMax_A
  close(7)
  
  if(nTraining_B<1000) then
     write (filename, '( "TrainingData/",A3,"RegionD_HyperParams_Symm", I3.3, ".dat" )' ) qualityString, nTraining_B
  else
     write (filename, '( "TrainingData/",A3,"RegionD_HyperParams_Symm", I4.4, ".dat" )' ) qualityString, nTraining_B
  endif
  filename = trim(codedir) // filename
  open (unit = 7, file = filename)
  do i=1,nDim
     read (7,*) dum
     j=int(dum)
     read (7,*) lScale_B(j)
  end do
  read (7,*) expVar_B
  read (7,*) NuggVar_B
  read (7,*) gpEMax_B
  close(7)


  if(nTraining_C<1000) then
     write (filename, '( "TrainingData/",A3,"RegionE_HyperParams_Symm", I3.3, ".dat" )' ) qualityString, nTraining_C
  else
     write (filename, '( "TrainingData/",A3,"RegionE_HyperParams_Symm", I4.4, ".dat" )' )  qualityString, nTraining_C
  endif
  filename = trim(codedir) // filename
  open (unit = 7, file = filename)
  do i=1,nDim
     read (7,*) dum
     j=int(dum)
     read (7,*) lScale_C(j)
  end do
  read (7,*) expVar_C
  read (7,*) NuggVar_C
  read (7,*) gpEMax_C
  close(7)
  
  
  !====Load alpha coefficients====
  if(nTraining_A<1000) then
     write (filename, '( "TrainingData/",A3,"RegionA_alpha_Symm", I3.3, ".dat" )' ) qualityString, nTraining_A
  else
     write (filename, '( "TrainingData/",A3,"RegionA_alpha_Symm", I4.4, ".dat" )' )  qualityString,nTraining_A
  endif
  filename = trim(codedir) // filename
  open (unit = 7, file = filename)
  do i=1,nTraining_A
     read (7,*) alpha_A(i)
    end do
  close(7)
  
  if(nTraining_B<1000) then
     write (filename, '( "TrainingData/",A3,"RegionD_alpha_Symm", I3.3, ".dat" )' ) qualityString, nTraining_B
  else
     write (filename, '( "TrainingData/",A3,"RegionD_alpha_Symm", I4.4, ".dat" )' ) qualityString, nTraining_B
  endif
  filename = trim(codedir) // filename
  open (unit = 7, file = filename)
  do i=1,nTraining_B
     read (7,*) alpha_B(i)
    end do
  close(7)

  if(nTraining_C<1000) then
     write (filename, '( "TrainingData/",A3,"RegionE_alpha_Symm", I3.3, ".dat" )' ) qualityString, nTraining_C
  else
     write (filename, '( "TrainingData/",A3,"RegionE_alpha_Symm", I4.4, ".dat" )' )  qualityString,nTraining_C
  endif
  filename = trim(codedir) // filename
  open (unit = 7, file = filename)
  do i=1,nTraining_C
     read (7,*) alpha_C(i)
    end do
  close(7)



  !====Load training data x values ====
  if(nTraining_A<1000) then
     write (filename, '( "TrainingData/",A3,"RegionA_xTraining", I3.3, ".dat" )' ) qualityString, nTraining_A
  else
     write (filename, '( "TrainingData/",A3,"RegionA_xTraining", I4.4, ".dat" )' ) qualityString, nTraining_A
  endif
  filename = trim(codedir) // filename
  open (unit = 7, file = filename)
    
  do i=1,nTraining_A
     read (7,*) xTraining_A(:,i)
  end do
  close(7)

  if(nTraining_B<1000) then
     write (filename, '( "TrainingData/",A3,"RegionD_xTraining", I3.3, ".dat" )' ) qualityString, nTraining_B
  else
     write (filename, '( "TrainingData/",A3,"RegionD_xTraining", I4.4, ".dat" )' ) qualityString, nTraining_B
  endif
  filename = trim(codedir) // filename
  open (unit = 7, file = filename)
    
  do i=1,nTraining_B
     read (7,*) xTraining_B(:,i)
  end do
  close(7)

  if(nTraining_C<1000) then
     write (filename, '( "TrainingData/",A3,"RegionE_xTraining", I3.3, ".dat" )' )  qualityString,nTraining_C
  else
     write (filename, '( "TrainingData/",A3,"RegionE_xTraining", I4.4, ".dat" )' ) qualityString, nTraining_C
  endif
  filename = trim(codedir) // filename
  open (unit = 7, file = filename)
    
  do i=1,nTraining_C
     read (7,*) xTraining_C(:,i)
  end do
  close(7)

  
  !====Load the permuations====
  write (filename, '( "2CO2-Ar.sym" )' )
  filename = trim(codedir) // filename
  open (unit = 7, file = filename)
  read(7,*) perm

  ! ====Check that the lscales agree when there should be a permutation====
  do j=1,nDim
     do i=1,nPerms
        if( (lScale_A(perm(j,i))-lScale_A(j))**2 > 1e-10 .OR. (lScale_B(perm(j,i))-lScale_B(j))**2  > 1e-10 &
           .OR. (lScale_C(perm(j,i))-lScale_C(j))**2 > 1e-10) then
           print *,"Length scales do not agree when the sym files says that they should"
           print *, j, lScale_A(j), perm(j,i), lScale_A(perm(j,i)), lScale_A(j)-lScale_A(perm(j,i))
           stop 9
        end if
     enddo
  enddo
  
  !! Permute the training vectors
  do i=1,nDim
     do j=1,nPerms
        do k=1,nTraining_A
           xTrainingPerm_A(i,j,k)=xTraining_A(perm(i,j),k)
        end do
        do k=1,nTraining_B
           xTrainingPerm_B(i,j,k)=xTraining_B(perm(i,j),k)
        end do
        do k=1,nTraining_C
           xTrainingPerm_C(i,j,k)=xTraining_C(perm(i,j),k)
        end do
     end do
  end do

end subroutine load_GP_Data_2CO2_Ar
  

subroutine load_GP_Asymp_Data_2CO2_Ar
  use GP_variables_2CO2_Ar, ONLY: nDim, nPerms
  use GP_variables_Asympt_2CO2_Ar ! Data structure -  all variables needed
  use perm_array, only: perm
  
  integer i,j,k
  character (len=300) :: filename
  CHARACTER(len=300) :: homedir,codedir
  double precision dum

  allocate (perm(nDim,nPerms))
  
  allocate (alpha_A_Energy(nTraining_A_Energy), lScale_A_Energy(nDim), xTraining_A_Energy(nDim,nTraining_A_Energy))
  allocate (xTrainingPerm_A_Energy(nDim,nPerms,nTraining_A_Energy))

  allocate (alpha_D_Energy(nTraining_D_Energy), lScale_D_Energy(nDim), xTraining_D_Energy(nDim,nTraining_D_Energy))
  allocate (xTrainingPerm_D_Energy(nDim,nPerms,nTraining_D_Energy))
  
  allocate (alpha_E_Energy(nTraining_E_Energy), lScale_E_Energy(nDim), xTraining_E_Energy(nDim,nTraining_E_Energy))
  allocate (xTrainingPerm_E_Energy(nDim,nPerms,nTraining_E_Energy))

  
  CALL getenv("HOME", homedir)
  codedir=TRIM(homedir) // '/source/2CO2-Ar_PES/'


  
  !********************** ENERGY GP ***************************************************
  !====Load hyperparameters====
  if(nTraining_A_Energy<100) then
     write (filename, '( "TrainingData/",A3,"RegionA_Energy_HyperParams_Symm", I2.2, ".dat" )' ) &
          qualityString,  nTraining_A_Energy
  else
     if(nTraining_A_Energy<1000) then
        write (filename, '( "TrainingData/",A3,"RegionA_Energy_HyperParams_Symm", I3.3, ".dat" )' ) &
             qualityString,  nTraining_A_Energy
     else
        write (filename, '( "TrainingData/",A3,"RegionA_Energy_HyperParams_Symm", I4.4, ".dat" )' ) &
             qualityString,  nTraining_A_Energy
     endif
  endif
  
  filename = trim(codedir) // filename
  open (unit = 7, file = filename)
  do i=1,nDim
     read (7,*) dum
     j=int(dum)
     read (7,*) lScale_A_Energy(j)
  end do
  read (7,*) expVar_A_Energy
  read (7,*) NuggVar_A_Energy
  read (7,*) dum
  close(7)
  
  !====Load alpha coefficients====
  if(nTraining_A_Energy<100) then
     write (filename, '( "TrainingData/",A3,"RegionA_Energy_alpha_Symm", I2.2, ".dat" )' ) &
          qualityString,  nTraining_A_Energy
  else
     if(nTraining_A_Energy<1000) then
        write (filename, '( "TrainingData/",A3,"RegionA_Energy_alpha_Symm", I3.3, ".dat" )' ) &
               qualityString,  nTraining_A_Energy
     else
        write (filename, '( "TrainingData/",A3,"RegionA_Energy_alpha_Symm", I4.4, ".dat" )' ) &
             qualityString,  nTraining_A_Energy
     endif
  endif
  
  filename = trim(codedir) // filename
  open (unit = 7, file = filename)
  do i=1,nTraining_A_Energy
     read (7,*) alpha_A_Energy(i)
  end do
  close(7)

  !====Load training data x values ====
  if(nTraining_A_Energy<100) then
     write (filename, '( "TrainingData/",A3,"RegionA_Energy_xTraining", I2.2, ".dat" )' ) &
          qualityString,  nTraining_A_Energy
  else
     if(nTraining_A_Energy<1000) then
        write (filename, '( "TrainingData/",A3,"RegionA_Energy_xTraining", I3.3, ".dat" )' ) &
             qualityString,  nTraining_A_Energy
       else
          write (filename, '( "TrainingData/",A3,"RegionA_Energy_xTraining", I4.4, ".dat" )' ) &
               qualityString,  nTraining_A_Energy
       endif
    endif
              
    filename = trim(codedir) // filename
  open (unit = 7, file = filename)
  do i=1,nTraining_A_Energy
     read (7,*) xTraining_A_Energy(:,i)
  end do
  close(7)

   !====Load the permuations====
  write (filename, '( "2CO2-Ar.sym" )' )
  filename = trim(codedir) // filename
  open (unit = 7, file = filename)
  read(7,*) perm


  ! ====Check that the lscales agree when there should be a permutation====
  do j=1,nDim
     do i=1,nPerms
        if( (lScale_A_Energy(perm(j,i))-lScale_A_Energy(j))**2 > 1e-10) then
           print *,"Length scales do not agree when the sym files says that they should"
           print *, j, lScale_A_Energy(j), perm(j,i), lScale_A_Energy(perm(j,i)), &
                lScale_A_Energy(j)-lScale_A_Energy(perm(j,i))
           stop 9
        end if
     enddo
  enddo
  
  !! Permute the training vectors
  do i=1,nDim
     do j=1,nPerms
        do k=1,nTraining_A_Energy
           xTrainingPerm_A_Energy(i,j,k)=xTraining_A_Energy(perm(i,j),k)
        end do
     end do
  end do

  !==============================================Region D======================================

    !====Load hyperparameters====
  if(nTraining_D_Energy<100) then
     write (filename, '( "TrainingData/",A3,"RegionD_Energy_HyperParams_Symm", I2.2, ".dat" )' ) &
          qualityString,  nTraining_D_Energy
  else
     if(nTraining_D_Energy<1000) then
        write (filename, '( "TrainingData/",A3,"RegionD_Energy_HyperParams_Symm", I3.3, ".dat" )' ) &
             qualityString,  nTraining_D_Energy
     else
        write (filename, '( "TrainingData/",A3,"RegionD_Energy_HyperParams_Symm", I4.4, ".dat" )' ) &
             qualityString,  nTraining_D_Energy
     endif
  endif
  
  filename = trim(codedir) // filename
  open (unit = 7, file = filename)
  do i=1,nDim
     read (7,*) dum
     j=int(dum)
     read (7,*) lScale_D_Energy(j)
  end do
  read (7,*) expVar_D_Energy
  read (7,*) NuggVar_D_Energy
  read (7,*) dum
  close(7)
  !====Load alpha coefficients====
  if(nTraining_D_Energy<100) then
     write (filename, '( "TrainingData/",A3,"RegionD_Energy_alpha_Symm", I2.2, ".dat" )' ) &
          qualityString,  nTraining_D_Energy
  else
     if(nTraining_D_Energy<1000) then
        write (filename, '( "TrainingData/",A3,"RegionD_Energy_alpha_Symm", I3.3, ".dat" )' ) &
               qualityString,  nTraining_D_Energy
     else
        write (filename, '( "TrainingData/",A3,"RegionD_Energy_alpha_Symm", I4.4, ".dat" )' ) &
             qualityString,  nTraining_D_Energy
     endif
  endif
  
  filename = trim(codedir) // filename
  open (unit = 7, file = filename)
  do i=1,nTraining_D_Energy
     read (7,*) alpha_D_Energy(i)
  end do
  close(7)

  !====Load training data x values ====
  if(nTraining_D_Energy<100) then
     write (filename, '( "TrainingData/",A3,"RegionD_Energy_xTraining", I2.2, ".dat" )' ) &
          qualityString,  nTraining_D_Energy
  else
     if(nTraining_D_Energy<1000) then
        write (filename, '( "TrainingData/",A3,"RegionD_Energy_xTraining", I3.3, ".dat" )' ) &
             qualityString,  nTraining_D_Energy
       else
          write (filename, '( "TrainingData/",A3,"RegionD_Energy_xTraining", I4.4, ".dat" )' ) &
               qualityString,  nTraining_D_Energy
       endif
    endif
              
    filename = trim(codedir) // filename
  open (unit = 7, file = filename)
  do i=1,nTraining_D_Energy
     read (7,*) xTraining_D_Energy(:,i)
  end do
  close(7)

   !====Load the permuations====
  write (filename, '( "2CO2-Ar.sym" )' )
  filename = trim(codedir) // filename
  open (unit = 7, file = filename)
  read(7,*) perm


  ! ====Check that the lscales agree when there should be a permutation====
  do j=1,nDim
     do i=1,nPerms
        if( (lScale_D_Energy(perm(j,i))-lScale_D_Energy(j))**2 > 1e-10) then
           print *,"Length scales do not agree when the sym files says that they should"
           print *, j, lScale_D_Energy(j), perm(j,i), lScale_D_Energy(perm(j,i)), &
                lScale_D_Energy(j)-lScale_D_Energy(perm(j,i))
           stop 9
        end if
     enddo
  enddo
  
  !! Permute the training vectors
  do i=1,nDim
     do j=1,nPerms
        do k=1,nTraining_D_Energy
           xTrainingPerm_D_Energy(i,j,k)=xTraining_D_Energy(perm(i,j),k)
        end do
     end do
  end do

  
  !===========================Region E============================================

    !====Load hyperparameters====
  if(nTraining_E_Energy<100) then
     write (filename, '( "TrainingData/",A3,"RegionE_Energy_HyperParams_Symm", I2.2, ".dat" )' ) &
          qualityString,  nTraining_E_Energy
  else
     if(nTraining_E_Energy<1000) then
        write (filename, '( "TrainingData/",A3,"RegionE_Energy_HyperParams_Symm", I3.3, ".dat" )' ) &
             qualityString,  nTraining_E_Energy
     else
        write (filename, '( "TrainingData/",A3,"RegionE_Energy_HyperParams_Symm", I4.4, ".dat" )' ) &
             qualityString,  nTraining_E_Energy
     endif
  endif
  
  filename = trim(codedir) // filename
  open (unit = 7, file = filename)
  do i=1,nDim
     read (7,*) dum
     j=int(dum)
     read (7,*) lScale_E_Energy(j)
  end do
  read (7,*) expVar_E_Energy
  read (7,*) NuggVar_E_Energy
  read (7,*) dum
  close(7)
  !====Load alpha coefficients====
  if(nTraining_E_Energy<100) then
     write (filename, '( "TrainingData/",A3,"RegionE_Energy_alpha_Symm", I2.2, ".dat" )' ) &
          qualityString,  nTraining_E_Energy
  else
     if(nTraining_E_Energy<1000) then
        write (filename, '( "TrainingData/",A3,"RegionE_Energy_alpha_Symm", I3.3, ".dat" )' ) &
               qualityString,  nTraining_E_Energy
     else
        write (filename, '( "TrainingData/",A3,"RegionE_Energy_alpha_Symm", I4.4, ".dat" )' ) &
             qualityString,  nTraining_E_Energy
     endif
  endif
  
  filename = trim(codedir) // filename
  open (unit = 7, file = filename)
  do i=1,nTraining_E_Energy
     read (7,*) alpha_E_Energy(i)
  end do
  close(7)

  !====Load training data x values ====
  if(nTraining_E_Energy<100) then
     write (filename, '( "TrainingData/",A3,"RegionE_Energy_xTraining", I2.2, ".dat" )' ) &
          qualityString,  nTraining_E_Energy
  else
     if(nTraining_E_Energy<1000) then
        write (filename, '( "TrainingData/",A3,"RegionE_Energy_xTraining", I3.3, ".dat" )' ) &
             qualityString,  nTraining_E_Energy
       else
          write (filename, '( "TrainingData/",A3,"RegionE_Energy_xTraining", I4.4, ".dat" )' ) &
               qualityString,  nTraining_E_Energy
       endif
    endif
              
    filename = trim(codedir) // filename
  open (unit = 7, file = filename)
  do i=1,nTraining_E_Energy
     read (7,*) xTraining_E_Energy(:,i)
  end do
  close(7)

   !====Load the permuations====
  write (filename, '( "2CO2-Ar.sym" )' )
  filename = trim(codedir) // filename
  open (unit = 7, file = filename)
  read(7,*) perm


  ! ====Check that the lscales agree when there should be a permutation====
  do j=1,nDim
     do i=1,nPerms
          if( (lScale_E_Energy(perm(j,i))-lScale_E_Energy(j))**2 > 1e-10) then 
           print *,"Length scales do not agree when the sym files says that they should"
           print *, j, lScale_E_Energy(j), perm(j,i), lScale_E_Energy(perm(j,i)), &
                lScale_E_Energy(j)-lScale_E_Energy(perm(j,i))
           stop 9
        end if
     enddo
  enddo
  
  !! Permute the training vectors
  do i=1,nDim
     do j=1,nPerms
        do k=1,nTraining_E_Energy
           xTrainingPerm_E_Energy(i,j,k)=xTraining_E_Energy(perm(i,j),k)
        end do
     end do
  end do

  
  
end subroutine load_GP_Asymp_Data_2CO2_Ar

end module loadData_2CO2_Ar
