module testing_2CO2_Ar
  use AnglesStructures, only: Angles
  use CentreToCentreTuple_Module, only: CentreToCentreTuple
  use Overall_PES_2CO2_Ar, only: PES_2CO2_Ar
  use compDist_2CO2_Ar, only: computeDistances
  use compAngles_2CO2_Ar, only:computeAngles
  use GP_variables_2CO2_Ar, only: nDim
  use permutations, only: fixpermutations
  use inSituChecks, only: testPerms, arePermsTheSame
  implicit none
  private

  !!====Public interface================================================
  public :: fixedAngleSlice, randomPointTest, EvsE,  &
       scanThroughPointsEnergyTest, scanThroughPointsPermutationsTest
  !!====================================================================


  
contains



subroutine fixedAngleSlice()
  double precision, dimension(nDim):: rab
  double precision e, cosTheta,x,rin
  double precision start,end
  integer i,itot, flag
  double precision:: cosAngle
  integer :: num_args, ix
  character(len=400), dimension(:), allocatable :: args
  character(len=400) :: outputFilename, extension
  type( Angles ) :: myAngles
  type(CentreToCentreTuple) :: rCentre
  
  
  !Get command line arguments
  num_args = command_argument_count()
  allocate(args(num_args))  
  do ix = 1, num_args
     call get_command_argument(ix,args(ix))
  end do

  if(num_args /= 8) then
     print *,'[Usage] 2CO2-Ar_PES.out [r (smallest dist)] [cosThetaHat/Bar] [cosBeta1]', &
          '[cosAlpha1] [cosBeta2] [cosAlpha1][0=region A/B; 1 region C;', &
          ' 2= alpha2 direct (regionA/B); 3= alpha2 direct (regionC)] [filename]'
     return
  endif

  !Read in the command line arguments
  read(args(1),*)rin
  read(args(2),*)cosTheta
  read(args(3),*)cosAngle
  myAngles%beta1=acos(cosAngle)  
  read(args(4),*)cosAngle
  myAngles%alpha1=acos(cosAngle)  
  read(args(5),*)cosAngle
  myAngles%beta2=acos(cosAngle)
  read(args(6),*)myAngles%alpha2
  read(args(7),*)flag

  extension = '.lhc'
  outputFilename = trim(args(8)) // trim(extension)

  if( flag>2) then
     !In this we read alpha2 directly so leave it as it is and reset the flag
     myAngles%alpha2 = myAngles%alpha2 / 180.0 * 4.0*ATAN(1.d0)
     flag= flag -2
  else
     !In this case alpha2<Pi and we've read cos(alpha2)
     myAngles%alpha2=acos(myAngles%alpha2)
  endif
  
  PRINT*, 'rin',rin,'cTh', cosTheta,'b1',myAngles%beta1,'a1',myAngles%alpha1,myAngles%beta2, &
       myAngles%alpha2, flag

    
  itot=200
  start=3.0
  end=30.8
  

  open (unit=15, file=outputFilename, status='replace')
  if (flag<0.5) then
     rCentre%r12 = rin 
     do i=0, itot
              
        x = (  start + (end-start)*i/(1.0*itot) ) 
        rCentre%r13=Sqrt( rCentre%r12**2/4.0  +  x**2  -  rCentre%r12*x*cosTheta )
        rCentre%r23=Sqrt( rCentre%r12**2/4.0  +  x**2  +  rCentre%r12*x*cosTheta )

        rab = computeDistances(rCentre,myAngles)
        e=PES_2CO2_Ar( rab)
        
#ifdef DEBUG
        call testPerms( rab)
#endif
        
        if( cosTheta>0.0) then
           write(15,*) 1.0/min(rab(10),rab(11), rab(12)), abs(e)
        endif
     enddo
  endif

  if (flag>0.5) then
     rCentre%r13=rin !set the smallest distance to rin (the command line argument)
     do i=0, itot
        
        ! specify centre-to-centre separation
        x = (  start + (end-start)*i/(1.0*itot) ) 
        rCentre%r12=Sqrt( rCentre%r13**2/4.0  +  x**2  -  rCentre%r13*x*cosTheta )
        rCentre%r23=Sqrt( rCentre%r13**2/4.0  +  x**2  +  rCentre%r13*x*cosTheta )
        
        rab = computeDistances(rCentre,myAngles)
        e=PES_2CO2_Ar( rab)
        
#ifdef DEBUG
        call testPerms( rab)
#endif
        
        write(15,*) 1.0/min(rab(1),rab(2), rab(3),rab(4),rab(5), rab(6),rab(7),rab(8), rab(9)) &
             , abs(e)
     enddo
  endif

  
  write(6,*)'Written to file: ',outputFilename
  close(15)

end subroutine fixedAngleSlice

  

  
subroutine EvsE()
  double precision, allocatable:: rab(:), xStar(:), dum(:)
  double precision:: NonAdd, funcVal, RMSE=0, mean=0
  integer:: i,j, count=0, nPoints

  allocate (rab(nDim), xStar(nDim), dum(nDim+5) )

  !====Read test data and compute error====
  open (unit = 7, file = "Dec17_CCSDT_rInv-10000.lhc")
  nPoints=10005

  open (unit=15, file="PES_Err.dat ", status='replace')

  do i=1,nPoints !!10078
        read(7,*) dum
        
     do j=1,nDim
        xStar(j)=dum(j)
     end do
     NonAdd=dum(nDim+5)

     rab(:) = 1/xStar(:)
     funcVal = PES_2CO2_Ar(rab)
#ifdef DEBUG
        call testPerms( rab)
#endif
     
     if( dum(nDim+1)<0.007 .AND. dum(nDim+2)<0.007 .AND. dum(nDim+3)<0.007 &
        ) then
        write(15,*) NonAdd, funcVal, NonAdd-funcVal
        RMSE = RMSE + (NonAdd-funcVal)**2
        mean = mean + (NonAdd)**2
        count = count +1
     endif
  enddo

  print *,Sqrt(RMSE/(1.0*count)), Sqrt(mean/(1.0*count)), Sqrt(RMSE/mean)*100, count
     
end subroutine EvsE

!   initialize a random seed from the system clock at every run (fortran 95 code)
subroutine init_random_seed()
  INTEGER :: i, n, clock
  INTEGER, DIMENSION(:), ALLOCATABLE :: seed
  CALL RANDOM_SEED(size = n)
  ALLOCATE(seed(n))
  CALL SYSTEM_CLOCK(COUNT=clock)
  seed = 0*clock + 37 * (/ (i - 1, i = 1, n) /)
  CALL RANDOM_SEED(PUT = seed)
    DEALLOCATE(seed)
  end subroutine init_random_seed


  subroutine scanThroughPointsEnergyTest(totalSteps)
    integer, intent(in):: totalSteps
    double precision, dimension(nDim) :: rab
    double precision xSep
    type( Angles ) :: myAngles
    type(CentreToCentreTuple) :: rCentre
    double precision:: cosThetaHat
    integer :: i,j,k,u,v,w,xi
    double precision, parameter :: PI = 4.d0*ATAN(1.d0)

    double precision:: r12min=1.d0, r12max=30.d0
    double precision:: cosThetaHatmin=-1.d0, cosThetaHatmax=1.d0
    double precision:: xmin=1.d0, xmax=30.d0
    !double precision:: beta1min=-0.1d0*PI, beta1max= 2.2d0*PI
    !double precision:: beta2min=-0.1d0*PI, beta2max= 2.2d0*PI
    !double precision:: alpha1min=-0.1d0*PI, alpha1max= 2.2d0*PI
    !double precision:: alpha2min=-0.1d0*PI, alpha2max= 2.2d0*PI
    double precision:: beta1min=0.d0, beta1max= 2.d0*PI
    double precision:: beta2min=0.d0, beta2max= 2.d0*PI
    double precision:: alpha1min=0.d0, alpha1max= 2.d0*PI
    double precision:: alpha2min=0.d0, alpha2max= 2.d0*PI
    double precision:: Energy

    do i=0, totalSteps
       do j=0, totalSteps
          print *,'j=',j
          do k=0, totalSteps
             do u=0, totalSteps
                do v=0, totalSteps
                   do w=0, totalSteps
                      do xi=0, totalSteps
                   
                         !print *,i,j,k,u,v,w,xi
       rCentre%r12 = r12min  +  (r12max - r12min)                         * i/totalSteps
       cosThetaHat = cosThetaHatmin  +  (cosThetaHatmax - cosThetaHatmin) * j/totalSteps
       myAngles%beta1 = beta1min  +  (beta1max - beta1min)                  * k/totalSteps
       myAngles%beta2 = beta2min  +  (beta2max - beta2min)                  * u/totalSteps
       myAngles%alpha1 = alpha1min  +  (alpha1max - alpha1max)              * v/totalSteps
       myAngles%alpha2 = alpha2min  +  (alpha2max - alpha2max)              * w/totalSteps
       xSep      =    xmin   +   (xmax - xmin)                            * xi/totalSteps

       rCentre%r13=Sqrt( rCentre%r12**2/4.0  +  xSep**2  -  rCentre%r12*xSep*cosThetaHat )
       rCentre%r23=Sqrt( rCentre%r12**2/4.0  +  xSep**2  +  rCentre%r12*xSep*cosThetaHat )

       myAngles%cosTheta23 = (rCentre%r12**2/4 + rCentre%r13**2 - xSep**2) / rCentre%r12/rCentre%r13

       rab = computeDistances(rCentre, myAngles )
       !call testPerms( rab)

       Energy = PES_2CO2_Ar( rab)
       
    enddo
 enddo
enddo
enddo
enddo

enddo
       if (MOD(i, 1)==0) then
          print *,'step,size, percent complete',i,totalSteps,1.0*i/(1.0*totalSteps)*100
       end if

enddo

end subroutine scanThroughPointsEnergyTest

  subroutine scanThroughPointsPermutationsTest(totalSteps)
    integer, intent(in):: totalSteps
    double precision, dimension(nDim) :: rab
    double precision xSep
    type( Angles ) :: myAngles
    type(CentreToCentreTuple) :: rCentre
    double precision:: cosThetaHat
    integer :: i,j,k,u,v,w,xi
    double precision, parameter :: PI = 4.d0*ATAN(1.d0)

    double precision:: r12min=1.d0, r12max=30.d0
    double precision:: cosThetaHatmin=-1.d0, cosThetaHatmax=1.d0
    double precision:: xmin=1.d0, xmax=30.d0
    double precision:: beta1min=-0.1d0*PI, beta1max= 2.2d0*PI
    double precision:: beta2min=-0.1d0*PI, beta2max= 2.2d0*PI
    double precision:: alpha1min=-0.1d0*PI, alpha1max= 2.2d0*PI
    double precision:: alpha2min=-0.1d0*PI, alpha2max= 2.2d0*PI

    do i=0, totalSteps
       do j=0, totalSteps
          do k=0, totalSteps
             do u=0, totalSteps
                do v=0, totalSteps
                   do w=0, totalSteps
                      do xi=0, totalSteps
                   
       rCentre%r12 = r12min  +  (r12max - r12min)                         * i/totalSteps
       cosThetaHat = cosThetaHatmin  +  (cosThetaHatmax - cosThetaHatmin) * j/totalSteps
       myAngles%beta1 = beta1min  +  (beta1max - beta1min)                  * k/totalSteps
       myAngles%beta2 = beta2min  +  (beta2max - beta2min)                  * u/totalSteps
       myAngles%alpha1 = alpha1min  +  (alpha1max - alpha1max)              * v/totalSteps
       myAngles%alpha2 = alpha2min  +  (alpha2max - alpha2max)              * w/totalSteps
       xSep      =    xmin   +   (xmax - xmin)                            * xi/totalSteps

       rCentre%r13=Sqrt( rCentre%r12**2/4.0  +  xSep**2  -  rCentre%r12*xSep*cosThetaHat )
       rCentre%r23=Sqrt( rCentre%r12**2/4.0  +  xSep**2  +  rCentre%r12*xSep*cosThetaHat )

       myAngles%cosTheta23 = (rCentre%r12**2/4 + rCentre%r13**2 - xSep**2) / rCentre%r12/rCentre%r13

       rab = computeDistances(rCentre, myAngles )
      
       call arePermsTheSame( rab )
      
    enddo
 enddo
enddo
enddo
enddo

enddo
enddo

end subroutine scanThroughPointsPermutationsTest

subroutine randomPointTest()
  double precision, dimension(nDim) :: rab
  double precision x
  type( Angles ) :: myAngles
  type(CentreToCentreTuple) :: rCentre
  double precision:: cosThetaHat
  integer :: step,steptot=500000000, pass
  double precision, parameter :: PI = 4.d0*ATAN(1.d0)

  double precision:: r12min=1.d0, r12max=30.d0
  double precision:: cosThetaHatmin=-1.d0, cosThetaHatmax=1.d0
  double precision:: xmin=1.d0, xmax=30.d0
  double precision:: beta1min=-0.1d0*PI, beta1max= 2.2d0*PI
  double precision:: beta2min=-0.1d0*PI, beta2max= 2.2d0*PI
  double precision:: alpha1min=-0.1d0*PI, alpha1max= 2.2d0*PI
  double precision:: alpha2min=-0.1d0*PI, alpha2max= 2.2d0*PI
  double precision :: rNum(7)

  CALL init_random_seed

  do pass=1,100
     do step=0, stepTot
        CALL RANDOM_NUMBER(rNum)
        rCentre%r12 = r12min  +  (r12max - r12min)                         * rNum(1)
        cosThetaHat = cosThetaHatmin  +  (cosThetaHatmax - cosThetaHatmin) * rNum(2)
        myAngles%beta1 = beta1min  +  (beta1max - beta1min)                  * rNum(3)
        myAngles%beta2 = beta2min  +  (beta2max - beta2min)                  * rNum(4)
        myAngles%alpha1 = alpha1min  +  (alpha1max - alpha1max)              * rNum(5)
        myAngles%alpha2 = alpha2min  +  (alpha2max - alpha2max)              * rNum(6)
        x      =    xmin   +   (xmax - xmin)                                *rNum(7)
        
        rCentre%r13=Sqrt( rCentre%r12**2/4.0  +  x**2  -  rCentre%r12*x*cosThetaHat )
        rCentre%r23=Sqrt( rCentre%r12**2/4.0  +  x**2  +  rCentre%r12*x*cosThetaHat )
        
        myAngles%cosTheta23 = (rCentre%r12**2/4 + rCentre%r13**2 - x**2) / rCentre%r12/rCentre%r13

        rab = computeDistances(rCentre, myAngles )
        rab = fixPermutations( rab )
        
        call calculateAnglesAndCheckDistances( rab )

        call testPerms( rab) 

        
        if (MOD(step, 500000)==0) then
           print *,'Pass ',pass,step,1.0*step/(1.0*stepTot)*100
        end if
     enddo
  enddo
  
   end subroutine randomPointTest
   

   subroutine calculateAnglesAndCheckDistances( rab )
     double precision, dimension(nDim), intent(in) :: rab

     double precision:: distanceTol=4d-5
     type( angles ) :: myAngles
     type(CentreToCentreTuple) :: rCentre
     double precision, dimension(nDim):: rabTest
     integer j

     myAngles = computeAngles(rab)
     rCentre = CentreToCentreTuple( rab(5), rab(11), rab(14))
     rabTest = computeDistances(rCentre,myAngles)
     
     do j=1,nDim
        if( abs(rab(j) - rabTest(j)) > distanceTol) then
           print *,'Failed to compute distances correctly'
           print *,'Input Angles: beta1', myAngles%beta1,'  alpha1',myAngles%alpha1,&
                '  beta2',myAngles%beta2,'  alpha2', myAngles%alpha2,' cosTheta23',&
                myAngles%cosTheta23
           print*,'Expected distances: ',rab
           print*,'Actual distances  : ',rabTest
           
           STOP
        endif
     end do
     
   end subroutine calculateAnglesAndCheckDistances
     
end module testing_2CO2_Ar
