function RegionB_Extrapolate( rab )
  use RegionA_Options !! Use options from Region A (eg max vectors, DeltaX, etc)
  use GP_variables
  use compDist
  use AnglesAndVectors
  use PES_details
  implicit none
  double precision RegionB_Extrapolate
  double precision, dimension(:) :: rab
  double precision, allocatable :: xStar(:) , rabnew(:),rabnewDelta(:)
  double precision r13new(3), r23new(3), r12new(3),r13newDelta(3), r23newDelta(3), r12newDelta(3)
  double precision scale1,  E2,E2Delta
  double precision cosBeta1,sinBeta1, cosAlpha1, sinAlpha1, cosTheta23, sinTheta23
  double precision cosThetaHat !![angle between Ar3 and centre of 1,2]
  double precision :: xnew = -1000 !! x [distance between 3 and centre of 1,2]
  double precision  gamma, debt
  allocate( xStar(nDim), rabnew(nDim),rabnewDelta(nDim))

  if( rab(2)< RShortMax*scaleFactor .and. rab(7)< RMedMax*scaleFactor.and. rab(5)< RLongMax*scaleFactor) then
     !print *,'Case B GP'
     xStar(:) = 1/rab(:)
     RegionB_Extrapolate = PES_GP( xStar)
     return;
  endif

  !print*,'============Case B============'
  call computeAngles(rab, cosBeta1,sinBeta1, cosAlpha1, sinAlpha1, cosTheta23, sinTheta23)
  !First scale down r12
  scale1=1.0
  if( rab(2) >RShortMax ) then 
     scale1=rab(2)/(RShortMax)
  endif
  r12new(1)=rab(2)/scale1
  r13new(1)=rab(5)/scale1
  r23new(1)=rab(7)/scale1
  !print*,'rnew(0): ',rab(2),rab(5),rab(7)
  !print*,'rnew(1): ', r12new(1),r13new(1),r23new(1)
  
  !If this is enough then apply scaling and return this value
  if( r23new(1)< rMedMax .and. r13new(1) < RLongMax) then
     call computeDistances(r12new(1),r13new(1),r23new(1),acos(cosAlpha1),acos(cosBeta1), rabnew)
     xStar = 1.0/rabnew
     RegionB_Extrapolate = PES_GP( xStar) * (1.0/scale1)**9
     return
  endif
  
  !!====Ok, we're going to have to slide along x until r23=RMedMax
  !! compute cosThetaHat (this is fixed throughout)
  cosThetaHat= (r23new(1)**2-r13new(1)**2)/r12new(1)/Sqrt(2.0*(r13new(1)**2+r23new(1)**2)-r12new(1)**2)
  
  !! Check whether a slide to r23=r13Min will make r13 small enough
  if( r23new(1) > RMedMax) then
     xnew= 0.5 *( -cosThetaHat*r12new(1) + Sqrt( r12new(1)**2*(cosThetaHat**2 - 1.0)  +  4.0 * RMedMax**2 ) )
     r13new(2) = Sqrt(r12new(1)**2/4.0 + xnew*xnew -r12new(1)*xnew*cosThetaHat)
     r23new(2) = RMedMax
  else
     r13new(2)=r13new(1)
     r23new(2)=r23new(1)
  endif
  
  if( r13new(2)> RLongMax) then
     debt = r13new(2)-RLongMax
     !print *,r13new(2), 'Debt=',debt
     scale1=  rab(2)/( r12new(1)-debt/2.0)
     r12new(1)=rab(2)/scale1
     r13new(1)=rab(5)/scale1
     r23new(1)=rab(7)/scale1
     !slide so that r13=maximum value
     xnew= 0.5 *( cosThetaHat*r12new(1) + Sqrt( r12new(1)**2*(-1.0 + cosThetaHat**2)  +  4.0 * RLongMax**2 ) )
  endif

  !! make the decided upon slide along x
  if( xnew >0.0) then
     r23new(2) = Sqrt(r12new(1)**2/4.0 + xnew*xnew + r12new(1)*xnew*cosThetaHat)
     r12new(2)=r12new(1)
     r13new(2) = Sqrt(r12new(1)**2/4.0 + xnew*xnew - r12new(1)*xnew*cosThetaHat)
  else
     print *,' xnew error in Region B'; error stop
  endif

  !! change x to x+Dx
  xnew = xnew + DeltaX
  r12newDelta(2)=r12new(1)
  r13newDelta(2) = Sqrt(r12new(1)**2/4.0 + xnew*xnew - r12new(1)*xnew*cosThetaHat)
  r23newDelta(2) = Sqrt(r12new(1)**2/4.0 + xnew*xnew + r12new(1)*xnew*cosThetaHat)

  !print*,'rnew(2): ', r12new(2),r13new(2),r23new(2)
  !print*,'rnewD2 : ', r12newDelta(2),r13newDelta(2),r23newDelta(2)

  !!Compute with GP using scaled distances then use power law to extrapolate to required distances for rab
  call computeDistances(r12new(2),r13new(2),r23new(2),acos(cosAlpha1),acos(cosBeta1), rabnew)
  xStar = 1.0/rabnew
  E2 = PES_GP( xStar) 

  call computeDistances(r12newDelta(2),r13newDelta(2),r23newDelta(2),acos(cosAlpha1),acos(cosBeta1), rabnew)
  xStar = 1.0/rabnew
  E2Delta = PES_GP( xStar) 

  gamma=min(gammaMax,log(E2/E2Delta)/log(r13new(2)/(r13newDelta(2))))
  !print*,gamma,scale1, E2*(1.0/scale1)**9

  RegionB_Extrapolate = (1.0/scale1)**9*E2*(r13new(2)/r13new(1))**(-gamma)
end function RegionB_Extrapolate
