function RegionC_Extrapolate( rab )
  use RegionC_Options 
  use GP_variables
  use compDist
  use AnglesAndVectors
  use PES_details
  implicit none
  double precision RegionC_Extrapolate
  double precision, dimension(:) :: rab
  double precision, allocatable :: xStar(:) , rabnew(:),rabnewDelta(:)
  double precision r13new(3), r23new(3), r12new(3),r13newDelta(3), r23newDelta(3), r12newDelta(3)
  double precision scale1,  E2,E2Delta
  double precision cosBeta1,sinBeta1, cosAlpha1, sinAlpha1, cosTheta23, sinTheta23
  double precision cosThetaXL! Angle between vector x and CO2 axis
  double precision cosThetaRL! Angle between vector Ar1-Ar2 and CO2 axis
  !double precision cosThetaHat !![angle between Ar3 and centre of 1,2]
    double precision cosThetaBar !![angle between x and line 2-3]
  double precision cosThetaTilde !![angle between x and line 1-2]
  double precision :: xnew , xoriginal !! x [distance between 3 and centre of 1,2]
  double precision  gamma, debt
  double precision Ar1_Ar2(3),C_O(3),  xVect(3)
  allocate( xStar(nDim), rabnew(nDim),rabnewDelta(nDim))

  !permute CO2 if it is flipped
  !if( (rab(1)< rab(3) .or. rab(4)<rab(6)) ) then
  !   xStar(:) = rab(:) !Careful this is not the intended use of xStar!
  !   rab(1)=xStar(3)
  !   rab(2)=xStar(2)
  !   rab(3)=xStar(1)
  !   rab(4)=xStar(6)
  !   rab(5)=xStar(5)
  !   rab(6)=xStar(4)
  !   rab(7)=xStar(7)
  !endif

  
  if( rab(7)< RShortMax*scaleFactor .and. rab(2)< RMedMax*scaleFactor .and. rab(5)< RLongMax*scaleFactor) then
     !print *,'Case C GP'
     xStar(:) = 1/rab(:)
     RegionC_Extrapolate = PES_GP( xStar)
     return;
  endif


  if( rab(7)<rab(2).AND.rab(2)<rab(5)) then !====CASE C====
     if( rab(7)< RShortMax*scaleFactor .AND. rab(5)< RLongMax*scaleFactor) then
        if(rab(1)< RMedMax.or. rab(2)< RMedMax.or. rab(3)< RMedMax ) then
           if( rab(4)< RLongMax.or.rab(5)< RLongMax.or.rab(6)< RLongMax) then
              !print *,'Case C GP - extra length from C-O bond means distances are short enough'
              xStar = 1.0/rab
              RegionC_Extrapolate = PES_GP( xStar) 
              return
           endif
        endif
     endif
     
    !print*,'============Case C============'
     call computeAngles(rab, cosBeta1,sinBeta1, cosAlpha1, sinAlpha1, cosTheta23, sinTheta23)

     
     !First scale down r23
     scale1=1.0
     if( rab(7) > rShortMax ) then
        scale1=rab(7)/rShortMax
     endif
     r12new(1)=rab(2)/scale1
     r13new(1)=rab(5)/scale1
     r23new(1)=rab(7)/scale1
     
     !print*,'rnew(0): ',rab(2),rab(5),rab(7)
     !print*,'rnew(1): ', r12new(1),r13new(1),r23new(1)
     
  
     
     !If this is enough then apply scaling and return this value
     call computeDistances(r12new(1),r13new(1),r23new(1),acos(cosAlpha1),acos(cosBeta1), rabnew)
     !print *,'rab (scale 1) : ', rabnew(1), rabnew(2), rabnew(3), rabnew(4),rabnew(5), rabnew(6)
     !print *,
     if(rabnew(1)< RMedMax.or. rabnew(2)< RMedMax.or. rabnew(3)< RMedMax ) then
        if( rabnew(4)< RLongMax.or.rabnew(5)< RLongMax.or.rabnew(6)< RLongMax) then
           xStar = 1.0/rabnew
           RegionC_Extrapolate = PES_GP( xStar) * (1.0/scale1)**9
           return
        endif
     endif

     !!====Ok, we're going to have to slide along x until min(r12)=r13MAX
     !! compute the following angles, which are fixed throughout
     cosThetaBar= (r13new(1)**2-r12new(1)**2)/r23new(1)/Sqrt(2.0*(r12new(1)**2+r13new(1)**2)-r23new(1)**2)
     
     Ar1_Ar2 = (/ r13new(1)*sinTheta23 ,  0.0D1 , r13new(1)*cosTheta23-r12new(1) /)
     C_O = (/sinBeta1*cosAlpha1, sinBeta1*sinAlpha1,cosBeta1/)
     xVect = (/ 0.0D1 ,  0.0D1 , r12new(1)/)   +   Ar1_Ar2/2.0
     
     xoriginal = sqrt(dotProd(xVect,xVect))
     cosThetaXL = dotProd(xVect, C_O) / xoriginal
     cosThetaRL = dotProd(Ar1_Ar2, C_O)/ sqrt(dotProd(Ar1_Ar2, Ar1_Ar2))
     cosThetaTilde = xVect(3) / xoriginal !!(xHat . zHat)

     !print *, 'RL=',cosThetaRL, ' XL=',cosThetaXL

     !! Find the slide that will make 1 of rab1,2 or 3 = RMedMax. NB: maximum x is the smaller slide
     xnew= Max( & 
          0.5 *( cosThetaBar*r23new(1) + Sqrt( r23new(1)**2*(-1.0 + cosThetaBar**2)  +  4.0 * RMedMax**2 ) ), &
          max( &
          0.5*(2.0*lCO*cosThetaXL+ r23new(1)*cosThetaBar &
          + sqrt( -4.0*lCO**2 + 4.0*RMedMax**2 -r23new(1)**2 &
          -4.0*r23new(1)*lCO*cosThetaRL+(2.0*lCO*cosThetaXL + r23new(1)*cosThetaBar)**2)), &
          0.5*(-2.0*lCO*cosThetaXL+ r23new(1)*cosThetaBar &
          + sqrt( -4.0*lCO**2 + 4.0*RMedMax**2 -r23new(1)**2 &
          +4.0*r23new(1)*lCO*cosThetaRL+(-2.0*lCO*cosThetaXL + r23new(1)*cosThetaBar)**2)) ) &
             )
     xnew= Min( xoriginal, xnew)
     r13new(2) = Sqrt(r23new(1)**2/4.0 + xnew*xnew + r23new(1)*xnew*cosThetaBar)
     r12new(2)= Sqrt(r23new(1)**2/4.0 + xnew*xnew - r23new(1)*xnew*cosThetaBar)
     r23new(2)=r23new(1)

     !print *,'x: ', xoriginal, xnew, xoriginal-xnew
     !print*,'rnew(1): ', r12new(1),r13new(1),r23new(1)
     !print*,'rnew(2): ', r12new(2),r13new(2),r23new(2)

     xoriginal = xoriginal *scale1 !Make xorginial truely original
     call computeDistancesWithSlide(r12new(1),r13new(1),r23new(1),acos(cosAlpha1),acos(cosBeta1), & 
          cosThetaTilde, xoriginal/scale1-xnew, rabnew)
     !!print *,'rab (slide 1) : ', rabnew(1), rabnew(2), rabnew(3)!, rab(4),rab(5), rab(6), r13MAX, r13MAX + r23MIN
     
     if( rabnew(4)> RLongMax .and. rabnew(5)> RLongMax .and. rabnew(6) >  RLongMax ) then
        debt = min(rabnew(4),rabnew(5),rabnew(6)) - RLongMax
        !!debt = rabnew(5) - r23MIN-r13MAX !!!CARE!!!!!!!!!!!!!!!!!!
        !print *,rabnew(4), rabnew(5),r13new(2)
        !print *,'Debt=',debt

       
        
        scale1=  rab(5)/( r13new(1)-debt/2.0)
        r12new(1)=rab(2)/scale1
        r13new(1)=rab(5)/scale1
        r23new(1)=rab(7)/scale1

        call computeDistances(r12new(1),r13new(1),r23new(1),acos(cosAlpha1),acos(cosBeta1), rabnew)
        !print*,'Scale 2: rnew(1): ', r12new(1),r13new(1),r23new(1)
        !!print *,'rabnew ', rabnew(4),rabnew(5), rabnew(6)
             
        !If this is enough then apply scaling and return this value
        if( rabnew(4)< RLongMax .or. rabnew(5) < RLongMax .or. rabnew(6) < RLongMax) then
           !print *, 'Returning value without sliding', scale1
           xStar = 1.0/rabnew
           RegionC_Extrapolate = PES_GP( xStar) * (1.0/scale1)**9
           return
        endif

        
        !redo the slide so that r13=maximum value
        xnew= Max( 0.5 *( &
             -cosThetaBar*r23new(1) + Sqrt(r23new(1)**2*(-1.0 + cosThetaBar**2) + 4.0 * RLongMax**2 ) &
             ), &
             0.5*(2.0*lCO*cosThetaXL - r23new(1)*cosThetaBar &
             + sqrt( 4.0*lCO**2*(CosThetaXL**2-1.0)  &
             +4.0*(cosThetaRL-CosThetaBar*cosThetaXL)*lCO*r23new(1) &
             +r23new(1)**2*(cosThetaBar**2-1.0) &
             + 4.0*RLongMax**2) ), &
             0.5*(-2.0*lCO*cosThetaXL - r23new(1)*cosThetaBar &
             + sqrt( 4.0*lCO**2*(CosThetaXL**2-1.0)  &
             +4.0*(-cosThetaRL+CosThetaBar*cosThetaXL)*lCO*r23new(1) &
             +r23new(1)**2*(cosThetaBar**2-1.0) &
             + 4.0*RLongMax**2) ) &
             )

        !print *,'Delta X',xoriginal/scale1-xnew, xnew, xoriginal
        call computeDistancesWithSlide(r12new(1),r13new(1),r23new(1),acos(cosAlpha1),acos(cosBeta1), & 
             cosThetaTilde, xoriginal/scale1-xnew, rabnew)
        !print*,'Slide 2 rab(4,5,6): ', rabnew(4), rabnew(5), rabnew(7)
        !xoriginal = xoriginal /scale1 !Revert xorginial 
     endif
     
     
     
     !! make the decided upon slide along x
     r23new(2)=r23new(1)
     r12new(2) = Sqrt(r23new(1)**2/4.0 + xnew*xnew - r23new(1)*xnew*cosThetaBar)
     r13new(2) = Sqrt(r23new(1)**2/4.0 + xnew*xnew + r23new(1)*xnew*cosThetaBar)

     
     
     !!Compute with GP using scaled distances then use power law to extrapolate to required distances for rab
     !!print *, 'Original cThetaT',  cosThetaTilde, xoriginal-xnew

     !print*,'rnew(1): ', r12new(1),r13new(1),r23new(1)
     !print*,'rnew(2): ', r12new(2),r13new(2),r23new(2), rabnew(2), rabnew(5), rabnew(7)


    
     !print *, 'r1-7 newX',rabnew(1), rabnew(2), rabnew(3),'; ', rabnew(4), rabnew(5), rabnew(6),'; ',rabnew(7)
     !print *,'rnew(2): ',rabnew(2), rabnew(5), rabnew(7)
    
     xStar = 1.0/rabnew
     E2 = PES_GP( xStar) 
     
     
     !! change x so that r12=r13MAX+0.1
     xnew = xnew + DeltaX
     r23newDelta(2)=r23new(1)
     r12newDelta(2) = Sqrt(r23new(1)**2/4.0 + xnew*xnew - r23new(1)*xnew*cosThetaBar)
     r13newDelta(2) = Sqrt(r23new(1)**2/4.0 + xnew*xnew + r23new(1)*xnew*cosThetaBar)
     
     !print*,'rnewD2 : ', r12newDelta(2),r13newDelta(2),r23newDelta(2)!,cosThetaBar, &
     !     (r13newDelta(2)**2-r12newDelta(2)**2)/r23newDelta(2)/Sqrt(2.0*(r12newDelta(2)**2+r13newDelta(2)**2)-r23newDelta(2)**2)

     !!Compute with GP using scaled distances then use power law to extrapolate to required distances for rab
     call computeDistancesWithSlide(r12new(1),r13new(1),r23new(1),acos(cosAlpha1),acos(cosBeta1), & 
          cosThetaTilde, xoriginal/scale1-xnew, rabnew)
     !print *,'rnewDelta(s): ',r12newDelta(2), r13newDelta(2), r23newDelta(2), rabnew(2), rabnew(5), rabnew(7)
     !print *, 'r1-7 Delta ',rabnew(1), rabnew(2), rabnew(3),'; ', rabnew(4), rabnew(5), rabnew(6),'; ',rabnew(7)

     xStar = 1.0/rabnew
     E2Delta = PES_GP( xStar) 
     
     gamma=min(gammaMax,log(E2/E2Delta)/log(r12new(2)/(r12newDelta(2))))
     
     !print*,gamma,scale1, E2*(1.0/scale1)**9, E2,E2Delta
     
     !print *,'rnew(2)     : ', r12new(2),r13new(2),r23new(2)
     !print *,'rnewDelta(s): ',r12newDelta(2), r13newDelta(2), r23newDelta(2)
     RegionC_Extrapolate = (1.0/scale1)**9*E2*(r12new(2)/r12new(1))**(-gamma)
     return
  endif
  
  ERROR STOP
  
end function RegionC_Extrapolate
